/*
    WaveDataCmd.h - supplementary wave data viewer (time format)

    Will Eighteen, 2005, 2006: will@opaline.demon.co.uk
*/


#ifndef WAVEDATACMD_H
#define WAVEDATACMD_H

#include "NoUndoCmd.h"
#include <Xm/Xm.h>
#include "wave/waveclass.h"
#include "MixwaveUtils.h"


typedef void ( *WaveDataCallback ) ( void *, void *, long, long, long, long,
					long, long, Boolean, unsigned, unsigned );

class WaveDataCmd : public NoUndoCmd
{
	public:
		WaveDataCmd ( char *, int active, WaveDataCallback, Wave *, int *, int, long, long, void * );
		WaveDataCmd::~WaveDataCmd();
		virtual const char *const className() { return ( "WaveDataCmd" ); }
		void setFields ( long, long, long, long, long, long, long );
		void setDatumValueBounds ( long, long );
		void setFPDigits ( int num ) { _fpDigits = num; };
		unsigned waveDataNum() { return _waveDataNumber; };
		void waveDataNum ( int );
		void cancel();
		void setArrayPtr ( int *ptr ) { _data = ptr; };

	protected:
		void doCommand();			// called by base class
		WaveDataCallback _callback;	// function to be called
		void *_clientData;			// data provided by caller

		// fns called when data in text box is changed
		virtual void field1Changed ( char * );
		virtual void field2Changed ( char * );
		virtual void field3Changed ( char * );
		virtual void field4Changed ( char * );
		virtual void field5Changed ( char * );

	private:
		/* control data for current view */
		long _field1Value, _field2Value, _field3Value;
		long _field4Value, _field5Value, _field7Value;
		float _field6Value;

		// callback for WM 'X'
		static void clientMessageProc ( Widget, XtPointer, XtPointer );

		// Callbacks for data change by user
		static void field1ChangedCallback ( Widget, XtPointer, XtPointer );
		static void field2ChangedCallback ( Widget, XtPointer, XtPointer );
		static void field3ChangedCallback ( Widget, XtPointer, XtPointer );
		static void field4ChangedCallback ( Widget, XtPointer, XtPointer );
		static void field5ChangedCallback ( Widget, XtPointer, XtPointer );

		static void field1EntryCallback ( Widget, XtPointer, XtPointer );
		static void field2EntryCallback ( Widget, XtPointer, XtPointer );
		static void field3EntryCallback ( Widget, XtPointer, XtPointer );
		static void field4EntryCallback ( Widget, XtPointer, XtPointer );
		static void field5EntryCallback ( Widget, XtPointer, XtPointer );
		static void field6EntryCallback ( Widget, XtPointer, XtPointer );

		static void reloadCallback ( Widget, XtPointer, XtPointer );

		// 'confirm new datum value' messageDialogManager callbacks
		static void datumChangeOkCallback ( XtPointer );
		static void datumChangeCancelCallback ( XtPointer );

		void reload();

		void changeField1 ( long );
		void changeField2 ( long );
		void changeField3 ( long );
		void changeField4 ( long );
		void changeField5 ( long );
		void changeField6 ( long );
		void changeField7 ( long );

		void field1Entry ( long );
		void field2Entry ( long );
		void field3Entry ( long );
		void field4Entry ( long );
		void field5Entry ( long );

		void datumChangeOk ( long );
		void checkSelection ( void );
		void clientMessage ( XmAnyCallbackStruct * );

		long datumNum ( long );
		long datumValue ( long );

		char _applicationName[128];
		char _name[128];
		long _sampleRate;
		long _maxData;				// max data in data source
		long _minY, _maxY;			// datum value limits
		long _numData;				// number of data points loaded into array
		long _startData;			// start of data in array (offset into file)
		long _oldStart, _oldEnd;	// preserve last used data range
		int *_data;					// data to be displayed
		unsigned int _numDigits;
		int _fpDigits;
		unsigned int _waveDataNumber;	// id of this graph
		unsigned int _callbackType;	// 0 = report/reload, 1 = 'ok', 2 = 'cancel' '
		MixwaveUtils *_utils;

		Widget _frame;
		Widget _popup, _rc0, _rca;
		Widget _field1; /* output area */
		Widget _field2;
		Widget _field3;
		Widget _field4;
		Widget _field5; /* current (default or pointer) x value */
		Widget _field6; /* current (default or pointer) y value */
		Widget _field7;

		Widget _label1; /* label for output area */
		Widget _label2;
		Widget _label3;
		Widget _label4;
		Widget _label5;
		Widget _label6;
		Widget _label7;

		/* command widgets */
		Widget _label;
};

#endif
